module TickTock

  class Duration

    MILLIS_IN_SECOND = 1000
    MILLIS_IN_MINUTE = MILLIS_IN_SECOND * 60
    MILLIS_IN_HOUR = MILLIS_IN_MINUTE * 60
    MILLIS_IN_DAY = MILLIS_IN_HOUR * 24
    MILLIS_IN_WEEK = MILLIS_IN_DAY * 7
    MILLIS_IN_YEAR = MILLIS_IN_WEEK * 52

    attr_reader :raw_millis

    def self.from_times(start, finish)
      Duration.new(((finish - start) * 1000).to_i)
    end

    def self.from_pieces(millis=0, seconds=0, minutes=0, hours=0, days=0, weeks=0, years=0)
      Duration.new(millis +
                   seconds * MILLIS_IN_SECOND +
                   minutes * MILLIS_IN_MINUTE +
                   hours * MILLIS_IN_HOUR +
                   days * MILLIS_IN_DAY +
                   weeks * MILLIS_IN_WEEK +
                   years * MILLIS_IN_YEAR)
    end

    def initialize(millis = 0)
      @raw_millis = millis
      @formatter = TickTock::Formatter.new(self)
    end

    def +(other)
      Duration.new(raw_millis + other.raw_millis)
    end

    def -(other)
      Duration.new(raw_millis - other.raw_millis)
    end

    def years
      raw_millis / MILLIS_IN_YEAR
    end

    def weeks
      (raw_millis - years_in_millis) / MILLIS_IN_WEEK
    end

    def days
      (raw_millis - years_in_millis - weeks_in_millis) / MILLIS_IN_DAY
    end

    def hours
      (raw_millis - years_in_millis - weeks_in_millis - days_in_millis) / MILLIS_IN_HOUR
    end

    def minutes
      (raw_millis - years_in_millis - weeks_in_millis - days_in_millis - hours_in_millis) / MILLIS_IN_MINUTE
    end

    def seconds
      (raw_millis - years_in_millis - weeks_in_millis - days_in_millis - hours_in_millis - minutes_in_millis) / MILLIS_IN_SECOND
    end

    def millis
      raw_millis - years_in_millis - weeks_in_millis - days_in_millis - hours_in_millis - minutes_in_millis - seconds_in_millis
    end

    def to_s
      @formatter.to_s
    end

    def inspect
      to_s
    end

    private

    def years_in_millis
      MILLIS_IN_YEAR * years
    end

    def weeks_in_millis
      MILLIS_IN_WEEK * weeks
    end

    def days_in_millis
      MILLIS_IN_DAY * days
    end

    def hours_in_millis
      MILLIS_IN_HOUR * hours
    end

    def minutes_in_millis
      MILLIS_IN_MINUTE * minutes
    end

    def seconds_in_millis
      MILLIS_IN_SECOND * seconds
    end

  end

end
