module TickTock
  class Formatter

    attr_reader :duration

    def initialize(duration)
      @duration = duration
    end

    def to_s
      "".tap do |s|
        s << " #{duration.years}y" if include_years?
        s << " #{duration.weeks}w" if include_weeks?
        s << " #{duration.days}d" if include_days?
        s << " #{duration.hours}h" if include_hours?
        s << " #{duration.minutes}m" if include_minutes?
        s << " #{duration.seconds}s" if include_seconds?
        s << " #{duration.millis}ms" if include_millis?
        s.lstrip!
      end
    end

    private

    def include_millis?
      true
    end

    def include_seconds?
      at_or_beyond? TickTock::Duration::MILLIS_IN_SECOND
    end

    def include_minutes?
      at_or_beyond? TickTock::Duration::MILLIS_IN_MINUTE
    end

    def include_hours?
      at_or_beyond? TickTock::Duration::MILLIS_IN_HOUR
    end

    def include_days?
      at_or_beyond? TickTock::Duration::MILLIS_IN_DAY
    end

    def include_weeks?
      at_or_beyond? TickTock::Duration::MILLIS_IN_WEEK
    end

    def include_years?
      at_or_beyond? TickTock::Duration::MILLIS_IN_YEAR
    end

    def at_or_beyond?(boundary)
      @duration.raw_millis >= boundary
    end

  end
end